@@ -54,6 +54,9 @@ feh_file *feh_file_new(char *filename)
 	else
 		newfile->name = estrdup(filename);
 	newfile->info = NULL;
+#ifdef HAVE_LIBEXIF
+	newfile->ed = NULL;
+#endif
 	return(newfile);
 }
 
@@ -70,7 +73,7 @@ void feh_file_free(feh_file * file)
 	if (file->info)
 		feh_file_info_free(file->info);
 #ifdef HAVE_LIBEXIF
-	if (file->info)
+	if (file->ed)
 		exif_data_unref(file->ed);
 #endif		
 	free(file);
@@ -212,15 +215,13 @@ void add_file_to_filelist_recursively(char *origpath, unsigned char level)
 		}
 		n = scandir(path, &de, file_selector_all, alphasort);
 		if (n < 0) {
-		  switch (errno) {
-		  case ENOMEM:
-		    if (!opt.quiet)
-		      weprintf("Insufficient memory to scan directory %s:", path);
-		    break;
-		  default:
-		    if (!opt.quiet)
-		      weprintf("Failed to scan directory %s:", path);
-		  }
+			switch (errno) {
+			case ENOMEM:
+				weprintf("Insufficient memory to scan directory %s:", path);
+				break;
+			default:
+				weprintf("Failed to scan directory %s:", path);
+			}
 		}
 
 		for (cnt = 0; cnt < n; cnt++) {
@@ -272,9 +273,6 @@ gib_list *feh_file_info_preload(gib_list * list)
 	feh_file *file = NULL;
 	gib_list *remove_list = NULL;
 
-	if (opt.verbose)
-		fputs(PACKAGE " - preloading...\n", stdout);
-
 	for (l = list; l; l = l->next) {
 		file = FEH_FILE(l->data);
 		D(("file %p, file->next %p, file->name %s\n", l, l->next, file->name));
@@ -458,6 +456,7 @@ gib_list *feh_read_filelist(char *filename)
 	char s[1024], s1[1024];
 	Imlib_Image tmp_im;
 	struct stat st;
+	signed short tmp_magick_timeout;
 
 	if (!filename)
 		return(NULL);
@@ -465,6 +464,8 @@ gib_list *feh_read_filelist(char *filename)
 	/*
 	 * feh_load_image will fail horribly if filename is not seekable
 	 */
+	tmp_magick_timeout = opt.magick_timeout;
+	opt.magick_timeout = -1;
 	if (!stat(filename, &st) && S_ISREG(st.st_mode) &&
 			feh_load_image_char(&tmp_im, filename)) {
 		weprintf("Filelist file %s is an image, refusing to use it.\n"
@@ -472,6 +473,7 @@ gib_list *feh_read_filelist(char *filename)
 		opt.filelistfile = NULL;
 		return NULL;
 	}
+	opt.magick_timeout = tmp_magick_timeout;
 
 	errno = 0;
 	if ((fp = fopen(filename, "r")) == NULL) {
@@ -530,7 +532,7 @@ void feh_save_filelist()
 
 	tmpname = feh_unique_filename("", "filelist");
 
-	if (!opt.quiet)
+	if (opt.verbose)
 		printf("saving filelist to filename '%s'\n", tmpname);
 
 	feh_write_filelist(filelist, tmpname);
