@@ -53,13 +53,13 @@ void init_parse_options(int argc, char **argv)
 	opt.display = 1;
 	opt.aspect = 1;
 	opt.slideshow_delay = 0.0;
+	opt.magick_timeout = -1;
 	opt.thumb_w = 60;
 	opt.thumb_h = 60;
 	opt.thumb_redraw = 10;
 	opt.menu_font = estrdup(DEFAULT_MENU_FONT);
 	opt.font = NULL;
 	opt.menu_bg = estrdup(PREFIX "/share/feh/images/menubg_default.png");
-	opt.menu_style = estrdup(PREFIX "/share/feh/fonts/menu.style");
 
 	opt.start_list_at = NULL;
 	opt.jump_on_resort = 1;
@@ -374,10 +374,10 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		{"bg-tile"       , 0, 0, 200},
 		{"bg-center"     , 0, 0, 201},
 		{"bg-scale"      , 0, 0, 202},
-		{"menu-style"    , 1, 0, 204},
 		{"zoom"          , 1, 0, 205},
 		{"no-screen-clip", 0, 0, 206},
 		{"index-info"    , 1, 0, 207},
+		{"magick-timeout", 1, 0, 208},
 		{"caption-path"  , 1, 0, 'K'},
 		{"action1"       , 1, 0, 209},
 		{"action2"       , 1, 0, 210},
@@ -590,7 +590,7 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 			}
 			break;
 		case 'R':
-			opt.reload = atoi(optarg);
+			opt.reload = atof(optarg);
 			break;
 		case 'a':
 			opt.alpha = 1;
@@ -627,11 +627,6 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		case 219:
 			opt.bgmode = BG_MODE_MAX;
 			break;
-		case 204:
-			free(opt.menu_style);
-			opt.menu_style = estrdup(optarg);
-			weprintf("The --menu-style option is deprecated and will be removed by 2012");
-			break;
 		case 205:
 			if (!strcmp("fill", optarg))
 				opt.zoom_mode = ZOOM_MODE_FILL;
@@ -646,6 +641,9 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		case 207:
 			opt.index_info = estrdup(optarg);
 			break;
+		case 208:
+			opt.magick_timeout = atoi(optarg);
+			break;
 		case 'K':
 			opt.caption_path = estrdup(optarg);
 			break;
