@@ -49,6 +49,9 @@ struct __fehoptions {
 	unsigned char jump_on_resort;
 	unsigned char full_screen;
 	unsigned char draw_filename;
+#ifdef HAVE_LIBEXIF
+	unsigned char draw_exif;
+#endif
 	unsigned char list;
 	unsigned char quiet;
 	unsigned char preload;
@@ -83,21 +86,18 @@ struct __fehoptions {
 	char *menu_font;
 	char *customlist;
 	char *menu_bg;
-	char *menu_style;
 	char *caption_path;
 	char *start_list_at;
 	char *info_cmd;
 	char *index_info;
 
-	gib_style *menu_style_l;
-
 	int force_aliasing;
 	int thumb_w;
 	int thumb_h;
 	int limit_w;
 	int limit_h;
 	unsigned int thumb_redraw;
-	int reload;
+	double reload;
 	int sort;
 	int debug;
 	int geom_flags;
@@ -114,6 +114,8 @@ struct __fehoptions {
 
 	double slideshow_delay;
 
+	signed short magick_timeout;
+
 	Imlib_Font menu_fn;
 };
 
@@ -163,6 +165,9 @@ struct __fehkb {
 	struct __fehkey render;
 	struct __fehkey toggle_actions;
 	struct __fehkey toggle_filenames;
+#ifdef HAVE_LIBEXIF
+	struct __fehkey toggle_exif;
+#endif
 	struct __fehkey toggle_info;
 	struct __fehkey toggle_pointer;
 	struct __fehkey toggle_aliasing;
